#!/usr/bin/perl -w

############################# Legal Stuff ##############################
# Copyright (C) 2004 Peter J. Hutnick
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to:
# Free Software Foundation, Inc.
# 59 Temple Place - Suite 330, Boston, MA  
# 02111-1307, US
#
# For information, write the author, Peter Hutnick via the web form at
# http://hutnick.com/feedback/ 
########################################################################

############################# Theory of Operation ######################
# This program doesn't do what you think it does.
#
# It converts /my/ LaTeX files to valid XHTML.  It converts /your/ LaTeX
# files to gibberish.
# 
# It is based on elementcheck.pl, my grammar checker.
########################################################################

########################## Algorithm ###################################
# 1. Load LaTeX file, loading \include and \input files recursively, 
# into one big, nasty LaTeX file.
# 
# 2. Scan through the file and get some info.  (E.g. title)
#
# 3. Do a search and replace on each patter in the DATA section.
#
# 4. Barf out some cookie-cutter XHTML.
#
# 5. Barf out the big, nasty LaTeX file, which has, hopefully, been 
#    transmuted into XHTML.
#
# 6. Barf out a little more cookie-cutter XHTML.
#  
########################################################################

# Safety First!
use strict;


# External Packages and such.
use Getopt::Std;
use vars qw($opt_h $opt_s $opt_f $opt_o);

# Here are the subs so we can check them before calling them.

sub loadrules(@) {

	my @rules = ();

	while (my $record = <DATA>) {
		unless ( $record =~ /^#/ or $record =~ /^\s/ ) {
			chomp $record;
			push @rules, $record;
		}
	}

	return @rules;
} # End sub loadrules.	




sub loadtext {

	my $text = "";
	open (INPUT, "< $opt_f")
	        or die "Can't open data file or no data file specified.\
	 (Forget the -f switch?)\n";

	{
	local $/ = undef; #Slurp in the whole file.
	$text = <INPUT>;
	}

	close (INPUT);

	return $text;

} #endsub loadtext




sub replace(@) {

	my @rules = @_;
	my $text = pop @rules;


	foreach my $rule (@rules) {
		eval("\$text =~ $rule");
	}

	return $text;


} # end sub replace.


sub output ($)
{

#	my $text = $_; # I'd love to know why this doesn't work.
	my $text = pop @_;

	open (OUTPUT, "> $opt_o");

print OUTPUT q(<!DOCTYPE html
     PUBLIC "-//W3C//DTD XHTML 1.0 Strict//EN"
     "http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd">


<html>
        <head>
                <meta http-equiv="Content-type" content='text/html;
                        charset="UTF-8"' />
                <title>Free Curriculum Project Homepage</title>


                <link rel="stylesheet" type="text/css"
                        href="./fcpstyle.css" />

        </head>

        <body>
);

	print OUTPUT $text;

print OUTPUT q(
                <hr />
                <p>
<a href="http://validator.w3.org/check/referer"><img
        src="http://www.w3.org/Icons/valid-xhtml10"
        alt="Valid XHTML 1.0!" height="31" width="88" />
</a>

<a href="http://jigsaw.w3.org/css-validator/check/referer"><img
        src="http://jigsaw.w3.org/css-validator/images/vcss"
        alt="Valid CSS!" />
</a>
                </p>
        </body>
</html>
);

} #end sub output.



# Begin the main program.

# Start dealing with the command line.

unless ( Getopt::Std::getopts('hsf:o:') )
	{ die "Use the -h switch for help."; }

# Let's get the legal stuff out of the way.
unless ( $opt_s ){
	print q(
platex version 0.0.a alpha, Copyright (C) 2004 Peter J. Hutnick,
platex comes with ABSOLUTELY NO WARRANTY; for details see the file
"COPYING" included with this program.

This is Free Software, and you are welcome to redistribute it under certain,
conditions listed in the file COPYING.

);
	}

if ( $opt_h )
	{ die q(h for help.  (You found it!)
f for filename of LaTeX file to be converted.
o for HTML output file.
s to suppress the copyright and NO WARRANTY statement.

If unsure, try "platex.pl -f [somefile.tex] -o [somefile.html]" 
);
	}



my @rules = loadrules;
unless ( scalar(@rules) ) { 
	die "No rules to check. (Missing rules file?)\n\n"; }


my $text = loadtext();

push @rules, $text; 	# I am growing to loath perl's handling of 
			# arguments.
$text = replace(@rules);

output($text);


__END__
# This DATA section contains the conversion patterns.

############ Begin Rules ###########


s/%.*?\n//g;

#badass
s/(http:\/\/.*?) /\<a href=\"$1\"\>$1\<\/a\> /g

s/\\\\/\<br \/\>/g
s/~ */&nbsp;/g
s/{} */&nbsp;/g;

s/--/&ndash;/g
s/---/&mdash;/g

s/\\copyright/&copy;/g
s/\$\^{tm}\$/&#153;/g

s/\\LaTeX/LaTeX/g
s/\\TeX/TeX/g
s/\\LyX/LyX/g

s/\\emph{(.*?)}/\<em\>$1\<\/em\>/g;

s/\\textbf{(.*?)}/\<b\>$1\<\/b\>/g;

s/\n\n/\n\<\/p\>\n\n\<p\>\n/g;

s/\\title{(.*)}/\<h1 class=\"center\"\>$1\<\/h1\>/g;
s/\\author{(.*)}/\<p\>\<em\>by $1\<\/em\>\<\/p\>/g;

# These all "eat" extraneous <p> and </p> tags.
s/\<p\>\n\\part{(.*)\n\<\/p\>}/\<h1\>$1\<\/h1\>/g;
s/\\chapter{(.*)}/\<h1\>$1\<\/h1\>/g;
s/\\chapter\*{(.*)}/\<h1\>$1\<\/h1\>/g;
s/\<p\>\n\\section{(.*)}\n\<\/p\>/\<h2\>$1\<\/h2\>/g;
s/\<p\>\n\\subsection{(.*)}\n\<\/p\>/\<h3\>$1\<\/h3\>/g;
s/\<p\>\n\\subsubsection{(.*)}\n\<\/p\>/\<h3\>$1\<\/h3\>/g;
s/\<p\>\n\\bigskip\n\<\/p\>/\<hr \/\>/g;


s/\\begin{description}/\<ul\>/g;
s/\\end{description}/\<\/ul\>/g;
s/\\begin{itemize}/\<ol\>/g;
s/\\end{itemize}/\<\/ol\>/g;
s/\\item/\<li\>/g;

s/\\usepackage.*?}//g; # Stingy glob.  "Missing" "{" catches [options].
s/\\begin{document}//g; # Might change this to <body> or something.
s/\\end{document}//g;   # Might change this to </body>.

s/\\make.*?\s//g;
s/\\set.*?\s//g;
s/\\new.*?\s//g;
s/\\provide.*?\s//g;
s/\\noindent.*?\s//g;

s/\\begin{comment}/\<!-- /g;
s/\\end{comment}/ --\>/g;

s/\\begin{quote}/\<blockquote\>/g;
s/\\end{quote}/\<\/blockquote\>/g;

s/\\begin{center}/\<div class=\"center\"\> /g;
s/\\end{center}/\<\/div\>/g;

s/\`\`/\<q\>/g;
s/''/\<\/q\>/g;

s/\\.*?}//g; # Remove any straggling commands.
s/{.*?}//g; # Remove any straggling {foo}


# Preserve this comment to make sure that the last rule's desc includes a \n.

